/*
 * datamodel.c: TWAMP datamodel plugin
 *
 * Copyright (C) 2022, IOPSYS Software Solutions AB.
 *
 * Author: amin.benramdhane@pivasoftware.com
 *
 * See LICENSE file for license related information.
 *
 */

#include <libbbfdm-api/dmcommon.h>
#include <libbbfdm-api/dmuci.h>
#include <libbbfdm_api.h>

#include "datamodel.h"

static int get_enabled_reflector_count(void)
{
	int count = 0;
	struct uci_section *s = NULL, *stmp = NULL;
	uci_foreach_sections_safe("twamp", "twamp_reflector", stmp, s) {
		char *enable = bbf_uci_get_value_by_section_fallback_def(s, "enable", "1");
		if (enable[0] == '1') {
			count = count + 1;
		}
	}

	return count;
}

/*************************************************************
* ENTRY METHOD
**************************************************************/
static int browseIPInterfaceTWAMPReflectorInst(struct dmctx *dmctx, DMNODE *parent_node, void *prev_data, char *prev_instance)
{
	char *inst = NULL;
	struct dmmap_dup *p = NULL;
	LIST_HEAD(dup_list);

	synchronize_specific_config_sections_with_dmmap_eq("twamp", "twamp_reflector", "dmmap_twamp", "interface",
							   section_name((struct uci_section *)prev_data), &dup_list);

	list_for_each_entry(p, &dup_list, list) {
		inst = handle_instance(dmctx, parent_node, p->dmmap_section, "twamp_inst", "twamp_alias");

		if (DM_LINK_INST_OBJ(dmctx, parent_node, (void *)p, inst) == DM_STOP)
			break;
	}

	free_dmmap_config_dup_list(&dup_list);
	return 0;
}

/*************************************************************
* ADD & DEL OBJ
**************************************************************/
static int addObjIPInterfaceTWAMPReflector(char *refparam, struct dmctx *ctx, void *data, char **instance)
{
	struct uci_section *connection = NULL, *dmmap_section = NULL;
	char name[42];

	snprintf(name, sizeof(name), "%s_%s", section_name((struct uci_section *)data), *instance);
	bbf_uci_add_section("twamp", "twamp_reflector", &connection);
	bbf_uci_rename_section(connection, name);

	bbf_uci_set_value_by_section(connection, "enable", "0");
	bbf_uci_set_value_by_section(connection, "interface", section_name((struct uci_section *)data));
	bbf_uci_set_value_by_section(connection, "port", "862");
	bbf_uci_set_value_by_section(connection, "max_ttl", "1");
	bbf_uci_set_value_by_section(connection, "port_list", "");
	bbf_uci_set_value_by_section(connection, "ip_list", "");

	if (get_enabled_reflector_count() == 0) {
		bbf_uci_set_value("twamp", "twamp", "enable", "0");
	} else {
		bbf_uci_set_value("twamp", "twamp", "enable", "1");
	}

	bbf_uci_add_section_bbfdm("dmmap_twamp", "twamp_reflector", &dmmap_section);
	bbf_uci_set_value_by_section(dmmap_section, "section_name", name);
	bbf_uci_set_value_by_section(dmmap_section, "twamp_inst", *instance);

	return 0;
}

static int delObjIPInterfaceTWAMPReflector(char *refparam, struct dmctx *ctx, void *data, char *instance, unsigned char del_action)
{
	struct uci_section *s = NULL, *stmp = NULL;

	switch (del_action) {
		case DEL_INST:
			bbf_uci_delete_section_by_section(((struct dmmap_dup *)data)->config_section, NULL, NULL);
			bbf_uci_delete_section_by_section(((struct dmmap_dup *)data)->dmmap_section, NULL, NULL);
			break;
		case DEL_ALL:
			uci_foreach_sections_safe("twamp", "twamp_reflector", stmp, s) {
				char *interface;

				bbf_uci_get_value_by_section(s, "interface", &interface);

				char *intf_sec = section_name((struct uci_section *)data);
				if (strcmp(interface, intf_sec) != 0) {
					continue;
				}

				struct uci_section *dmmap_section = NULL;
				get_dmmap_section_of_config_section("dmmap_twamp", "twamp", section_name(s), &dmmap_section);
				bbf_uci_delete_section_by_section(dmmap_section, NULL, NULL);
				bbf_uci_delete_section_by_section(s, NULL, NULL);
			}
			break;
	}

	if (get_enabled_reflector_count() == 0) {
		bbf_uci_set_value("twamp", "twamp", "enable", "0");
	}

	return 0;
}

/*************************************************************
* GET & SET PARAM
**************************************************************/
static int get_IPInterface_TWAMPReflectorNumberOfEntries(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
	int cnt = bbf_get_number_of_entries(ctx, data, instance, browseIPInterfaceTWAMPReflectorInst);
	dmasprintf(value, "%d", cnt);
	return 0;
}

static int get_IPInterfaceTWAMPReflector_Enable(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
	*value = bbf_uci_get_value_by_section_fallback_def(((struct dmmap_dup *)data)->config_section, "enable", "1");
	return 0;
}

static int configure_layers(struct dmmap_dup *data)
{
	char *interface = NULL;

	bbf_uci_get_value_by_section(data->config_section, "interface", &interface);
	if (interface != NULL && interface[0] == '\0') {
		return -1;
	}

	bbf_uci_set_value_by_section(data->config_section, "device", get_device(interface));
	return 0;
}

static int set_IPInterfaceTWAMPReflector_Enable(char *refparam, struct dmctx *ctx, void *data, char *instance, char *value, int action)
{
	bool b;

	switch (action)	{
		case VALUECHECK:
			if (bbfdm_validate_boolean(ctx, value))
				return FAULT_9007;
			break;
		case VALUESET:
			bbf_convert_string_to_bool(value, &b);
			if(b) {
				if (configure_layers((struct dmmap_dup *) data) == 0) {
					bbf_uci_set_value("twamp", "twamp", "enable", "1");
					bbf_uci_set_value_by_section(((struct dmmap_dup *)data)->config_section, "enable", "1");
				} else {
					return FAULT_9002;
				}
			} else {
				bbf_uci_set_value_by_section(((struct dmmap_dup *)data)->config_section, "enable", "0");

				if (get_enabled_reflector_count() == 0) {
					bbf_uci_set_value("twamp", "twamp", "enable", "0");
				}
			}
			break;
	}

	return 0;
}

static int get_IPInterfaceTWAMPReflector_Status(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
	char *enable;

	bbf_uci_get_value_by_section(((struct dmmap_dup *)data)->config_section, "enable", &enable);
	if (strcmp(enable, "1") == 0)
		*value = "Active";
	else
		*value = "Disabled";
	return 0;
}

static int get_IPInterfaceTWAMPReflector_Alias(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
	bbf_uci_get_value_by_section(((struct dmmap_dup *)data)->dmmap_section, "twamp_alias", value);
	if ((*value)[0] == '\0')
		dmasprintf(value, "cpe-%s", instance);
	return 0;
}

static int set_IPInterfaceTWAMPReflector_Alias(char *refparam, struct dmctx *ctx, void *data, char *instance, char *value, int action)
{
	switch (action)	{
		case VALUECHECK:
			if (bbfdm_validate_string(ctx, value, -1, 64, NULL, NULL))
				return FAULT_9007;
			break;
		case VALUESET:
			bbf_uci_set_value_by_section(((struct dmmap_dup *)data)->dmmap_section, "twamp_alias", value);
			break;
	}
	return 0;
}

static int get_IPInterfaceTWAMPReflector_Port(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
	*value = bbf_uci_get_value_by_section_fallback_def(((struct dmmap_dup *)data)->config_section, "port", "862");
	return 0;
}

static int set_IPInterfaceTWAMPReflector_Port(char *refparam, struct dmctx *ctx, void *data, char *instance, char *value, int action)
{
	switch (action)	{
		case VALUECHECK:
			if (bbfdm_validate_unsignedInt(ctx, value, RANGE_ARGS{{NULL,"65535"}}, 1))
				return FAULT_9007;
			break;
		case VALUESET:
			bbf_uci_set_value_by_section(((struct dmmap_dup *)data)->config_section, "port", value);
			break;
	}
	return 0;
}

static int get_IPInterfaceTWAMPReflector_MaximumTTL(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
	*value = bbf_uci_get_value_by_section_fallback_def(((struct dmmap_dup *)data)->config_section, "max_ttl", "1");
	return 0;
}

static int set_IPInterfaceTWAMPReflector_MaximumTTL(char *refparam, struct dmctx *ctx, void *data, char *instance, char *value, int action)
{
	switch (action)	{
		case VALUECHECK:
			if (bbfdm_validate_unsignedInt(ctx, value, RANGE_ARGS{{"1","255"}}, 1))
				return FAULT_9007;
			break;
		case VALUESET:
			bbf_uci_set_value_by_section(((struct dmmap_dup *)data)->config_section, "max_ttl", value);
			break;
	}
	return 0;
}

static int get_IPInterfaceTWAMPReflector_IPAllowedList(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
	bbf_uci_get_value_by_section(((struct dmmap_dup *)data)->config_section, "ip_list", value);
	return 0;
}

static int set_IPInterfaceTWAMPReflector_IPAllowedList(char *refparam, struct dmctx *ctx, void *data, char *instance, char *value, int action)
{
	switch (action)	{
		case VALUECHECK:
			if (bbfdm_validate_string_list(ctx, value, -1, -1, 255, -1, -1, NULL, NULL))
				return FAULT_9007;
			break;
		case VALUESET:
			bbf_uci_set_value_by_section(((struct dmmap_dup *)data)->config_section, "ip_list", value);
			break;
	}
	return 0;
}

static int get_IPInterfaceTWAMPReflector_PortAllowedList(char *refparam, struct dmctx *ctx, void *data, char *instance, char **value)
{
	bbf_uci_get_value_by_section(((struct dmmap_dup *)data)->config_section, "port_list", value);
	return 0;
}

static int set_IPInterfaceTWAMPReflector_PortAllowedList(char *refparam, struct dmctx *ctx, void *data, char *instance, char *value, int action)
{
	switch (action)	{
		case VALUECHECK:
			if (bbfdm_validate_string_list(ctx, value, -1, -1, 255, -1, -1, NULL, NULL))
				return FAULT_9007;
			break;
		case VALUESET:
			bbf_uci_set_value_by_section(((struct dmmap_dup *)data)->config_section, "port_list", value);
			break;
	}
	return 0;
}

/**********************************************************************************************************************************
*                                            OBJ & LEAF DEFINITION
***********************************************************************************************************************************/
/* ********** DynamicObj ********** */
DM_MAP_OBJ tDynamicObj[] = {
/* parentobj, nextobject, parameter */
{"Device.IP.Interface.{i}.", tDeviceTWAMPReflectorObj, tDeviceTWAMPReflectorParams},
{0}
};

/* *** Device.IP.Interface.{i}. *** */
DMOBJ tDeviceTWAMPReflectorObj[] = {
/* OBJ, permission, addobj, delobj, checkdep, browseinstobj, nextdynamicobj, dynamicleaf, nextobj, leaf, linker, bbfdm_type, uniqueKeys*/
{"TWAMPReflector", &DMWRITE, addObjIPInterfaceTWAMPReflector, delObjIPInterfaceTWAMPReflector, NULL, browseIPInterfaceTWAMPReflectorInst, NULL, NULL, NULL, tIPInterfaceTWAMPReflectorParams, NULL, BBFDM_BOTH, LIST_KEY{"Alias", "Port", NULL}},
{0}
};

DMLEAF tDeviceTWAMPReflectorParams[] = {
/* PARAM, permission, type, getvalue, setvalue, bbfdm_type*/
{"TWAMPReflectorNumberOfEntries", &DMREAD, DMT_UNINT, get_IPInterface_TWAMPReflectorNumberOfEntries, NULL, BBFDM_BOTH},
{0}
};

/* *** Device.IP.Interface.{i}.TWAMPReflector.{i}. *** */
DMLEAF tIPInterfaceTWAMPReflectorParams[] = {
/* PARAM, permission, type, getvalue, setvalue, bbfdm_type*/
{"Enable", &DMWRITE, DMT_BOOL, get_IPInterfaceTWAMPReflector_Enable, set_IPInterfaceTWAMPReflector_Enable, BBFDM_BOTH},
{"Status", &DMREAD, DMT_STRING, get_IPInterfaceTWAMPReflector_Status, NULL, BBFDM_BOTH},
{"Alias", &DMWRITE, DMT_STRING, get_IPInterfaceTWAMPReflector_Alias, set_IPInterfaceTWAMPReflector_Alias, BBFDM_BOTH},
{"Port", &DMWRITE, DMT_UNINT, get_IPInterfaceTWAMPReflector_Port, set_IPInterfaceTWAMPReflector_Port, BBFDM_BOTH},
{"MaximumTTL", &DMWRITE, DMT_UNINT, get_IPInterfaceTWAMPReflector_MaximumTTL, set_IPInterfaceTWAMPReflector_MaximumTTL, BBFDM_BOTH},
{"IPAllowedList", &DMWRITE, DMT_STRING, get_IPInterfaceTWAMPReflector_IPAllowedList, set_IPInterfaceTWAMPReflector_IPAllowedList, BBFDM_BOTH},
{"PortAllowedList", &DMWRITE, DMT_STRING, get_IPInterfaceTWAMPReflector_PortAllowedList, set_IPInterfaceTWAMPReflector_PortAllowedList, BBFDM_BOTH},
{0}
};
